==> la-src-0.5.tar.bz2: LogicAnalyzer/client/org/sump/analyzer/DeviceController.java <==
/*
 *  Copyright (C) 2006 Michael Poppitz
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 *
 */
package org.sump.analyzer;

import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.Timer;

// TODO: when the dialog is closed using the window decoration's close function, close() is not called

/**
 * GUI Component that allows the user to control the device and start captures.
 * <p>
 * Its modelled after JFileChooser and should allow for non-dialog implementations
 * making it somewhat reusable.
 * 
 * @version 0.5
 * @author Michael "Mr. Sump" Poppitz
 *
 */
public class DeviceController extends JComponent implements ActionListener, Runnable {
	public final static int ABORTED = 0;
	public final static int DATA_READ = 1;
	
	/**
	 * Creates an array of check boxes, adds it to the device controller and returns it.
	 * @param label label to use on device controller component
	 * @return array of created check boxes
	 */
	private JCheckBox[] createChannelList(String label) {
		JCheckBox[] boxes = new JCheckBox[32];

		for (int row = 0; row < 4; row++) {
			if (row == 0)
				add(new JLabel(label));
			else
				add(new JLabel());
	
			Container container = new Container();
			container.setLayout(new GridLayout(1, 8));
	
			for (int col = 0; col < 8; col++) {
				JCheckBox box = new JCheckBox();
				box.setEnabled(false);
				container.add(box);
				boxes[row * 8 + col] = box;
			}
	
			add(container);
		}
		return (boxes);
	}
	
	/**
	 * Constructs device controller component.
	 *
	 */
	public DeviceController() {
		super();
		setLayout(new GridLayout(17, 2, 5, 5));
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		device = new Device();

		String[] ports = Device.getPorts();
		portSelect = new JComboBox(ports);
		add(new JLabel("Analyzer Port:"));
		add(portSelect);
		
		String[] speeds = {
			"200MHz",
			"100MHz", "50MHz", "20MHz", "10MHz", "5MHz", "2MHz", "1MHz",
			"500kHz", "200kHz", "100kHz", "50kHz", "20kHz", "10kHz",
			"1kHz", "500Hz", "200Hz", "100Hz", "50Hz", "20Hz", "10Hz"
		};
		speedSelect = new JComboBox(speeds);
		speedSelect.addActionListener(this);
		add(new JLabel("Sampling Rate:"));
		add(speedSelect);

		Container groups = new Container();
		groups.setLayout(new GridLayout(1, 4));
		channelGroup = new JCheckBox[4];
		for (int i = 0; i < channelGroup.length; i++) {
			channelGroup[i] = new JCheckBox(Integer.toString(i));
			channelGroup[i].setSelected(true);
			groups.add(channelGroup[i]);
		}
		add(new JLabel("Channel Groups:"));
		add(groups);
		
		String[] sizes = {
			"256K", "128K", "64K", "32K", "16K", "8K", "4K",
			"2K", "1K", "512", "256", "128", "64"
		};
		sizeSelect = new JComboBox(sizes);
		sizeSelect.setSelectedIndex(7);
		add(new JLabel("Recording Size:"));
		add(sizeSelect);

		filterEnable = new JCheckBox("Enable");
		filterEnable.setSelected(true);
		filterEnable.setEnabled(false);
		add(new JLabel("Noise Filter: "));
		add(filterEnable);

		triggerEnable = new JCheckBox("Enable");
		triggerEnable.addActionListener(this);
		add(new JLabel("Trigger: "));
		add(triggerEnable);
		
		String[] ratios = {"0/100", "25/75", "50/50", "75/25", "100/0"};
		ratioSelect = new JComboBox(ratios);
		ratioSelect.setSelectedIndex(2);
		add(new JLabel("Before/After Ratio: "));
		add(ratioSelect);

		triggerMask = createChannelList("Trigger Channels (0..31):");
		triggerValue = createChannelList("Trigger Values (0..31):");
		
		progress = new JProgressBar(0, 100);
		add(new JLabel("Progress:"));
		add(progress);
		
		captureButton = new JButton("Capture");
		captureButton.addActionListener(this);
		add(captureButton);

		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		add(cancel);

		capturedData = null;
		timer = null;
		worker = null;
		status = ABORTED;
		
	}

	/**
	 * Internal method that initializes a dialog and add this component to it.
	 * @param frame owner of the dialog
	 */
	private void initDialog(JFrame frame) {
		// check if dialog exists with different owner and dispose if so
		if (dialog != null && dialog.getOwner() != frame) {
			dialog.dispose();
			dialog = null;
		}
		// if no valid dialog exists, create one
		if (dialog == null) {
			dialog = new JDialog(frame, "Capture", true);
			dialog.getContentPane().add(this);
			dialog.pack();
		}
		// reset progress bar
		progress.setValue(0);

		// sync dialog status with device
		updateFields();
	}

	/**
	 * Return the device data of the last successful run.
	 * 
	 * @return device data
	 */
	public CapturedData getDeviceData() {
		return (capturedData);
	}

	/**
	 * Extracts integers from strings regardless of trailing trash.
	 * 
	 * @param s string to be parsed
	 * @return integer value, 0 if parsing fails
	 */
	private int smartParseInt(String s) {
		int val = 0;
	
		try {
			for (int i = 1; i <= s.length(); i++)
				val = Integer.parseInt(s.substring(0, i));
		} catch (NumberFormatException E) {}
		
		return (val);
	}

	/**
	 * Starts capturing from device. Should not be called externally.
	 */
	public void run()  {
		String value = (String)portSelect.getSelectedItem();
		// TODO: need to check if attach was successful
		device.attach(value);
		
		value = (String)sizeSelect.getSelectedItem();
		int s = smartParseInt(value);
		if (value.indexOf("K") > 0)
			s *= 1024;
		System.out.println("Size: " + s);
		device.setSize(s);
		
		value = (String)ratioSelect.getSelectedItem();
		double r = 0.5;
		if (value.equals("100/0")) r = 0;
		else if (value.equals("25/75")) r = 0.75;
		else if (value.equals("50/50")) r = 0.5;
		else if (value.equals("75/25")) r = 0.25;
		else if (value.equals("0/100")) r = 1;
		System.out.println("Ratio: " + r);
		device.setRatio(r);

		device.setFilterEnabled(filterEnable.isSelected());
		int m = 0;
		int v = 0;
		for (int i = 0; i < 32; i++) {
			if (triggerMask[i].isSelected())
				m |= 1 << i;
			if (triggerValue[i].isSelected())
				v |= 1 << i;				
		}
		device.setTrigger(m, v);

		int enabledChannels = 0;
		for (int i = 0; i < channelGroup.length; i++)
			if (channelGroup[i].isSelected())
				enabledChannels |= 0xff << (8 * i);
		device.setEnabledChannels(enabledChannels);
		
		try {
			System.out.println("Run started");
			capturedData = device.run();
			System.out.println("Run completed");
			status = DATA_READ;
		} catch (Exception ex) {
			// TODO: could make sense to also return half read captures if array length is corrected
			capturedData = null;
			status = ABORTED;
			System.out.println("Run aborted");
			if (!(ex instanceof InterruptedException))
				ex.printStackTrace(System.out);
		}
		device.detach();
	}
	
	/**
	 * Sets the enabled state of all available trigger check boxes and the ratio select.
	 * @param enable <code>true</code> to enable trigger configuration fields, <code>false</code> to disable them
	 */
	private void setTriggerEnabled(boolean enable) {
		int channels = device.getAvailableChannelCount();
		
		for (int i = 0; i < channels; i++) {
			triggerMask[i].setEnabled(enable);
			triggerValue[i].setEnabled(enable);
		}
		for (int i = channels; i < 32; i++) {
			triggerMask[i].setEnabled(false);
			triggerValue[i].setEnabled(false);
		}
		ratioSelect.setEnabled(enable);
	}
	
	/**
	 * Sets the enabled state of all configuration components of the dialog.
	 * @param enable <code>true</code> to enable components, <code>false</code> to disable them
	 */
	private void setDialogEnabled(boolean enable) {
		triggerEnable.setEnabled(enable);
		captureButton.setEnabled(enable);
		portSelect.setEnabled(enable);
		speedSelect.setEnabled(enable);
		sizeSelect.setEnabled(enable);
		updateFields(enable);
	}

	/** activates / deactivates dialog options according to device status */
	public void updateFields() {
		updateFields(true);
	}

	/** activates / deactivates dialog options according to device status */
	private void updateFields(boolean enable) {
		triggerEnable.setSelected(device.isTriggerEnabled());
		if (triggerEnable.isSelected() || !enable)
			setTriggerEnabled(enable);
		setTriggerEnabled(device.isTriggerEnabled());
		filterEnable.setEnabled(device.isFilterAvailable() && enable);
		for (int i = 0; i < channelGroup.length; i++)
			channelGroup[i].setEnabled(enable && (i < device.getAvailableChannelCount() / 8));
	}
	
	/**
	 * Properly closes the dialog.
	 * This method makes sure timer and worker thread are stopped before the dialog is closed.
	 *
	 */
	private void close() {
		if (timer != null) {
			timer.stop();
			timer = null;
		}
		if (worker != null) {
			worker.interrupt();
			worker = null;
		}
		dialog.hide();
	}
	
	/**
	 * Starts the capture thread.
	 */
	private void startCapture() {
		try {
			setDialogEnabled(false);
			timer = new Timer(100, this);
			worker = new Thread(this);
			timer.start();
			worker.start();
		} catch(Exception E) {
			E.printStackTrace(System.out);
		}
	}
	
	/**
	 * Handles all action events for this component.
	 */ 
	public void actionPerformed(ActionEvent event) {
		Object o = event.getSource();
		String l = event.getActionCommand();
		
		if (o == timer) {
			if (status == DATA_READ) {
				close();
			} else {
				if(device.isRunning())
					progress.setValue(device.getPercentage());
			}
		} else {
		
			if (o == triggerEnable) {
				device.setTriggerEnabled(triggerEnable.isSelected());
				updateFields();
			} else if (o == speedSelect) {
				String value = (String)speedSelect.getSelectedItem();
				int f = smartParseInt(value);
				if (value.indexOf("M") > 0)
					f *= 1000000;
				else if (value.indexOf("k") > 0)
					f *= 1000;
				System.out.println("Rate: " + f);
				device.setRate(f);
				updateFields();
			} else if (l.equals("Capture")) {
				startCapture();
			} else {
				close();
			}
		}
	}
	
	/**
	 * Displays the device controller dialog with enabled configuration portion and waits for user input.
	 * 
	 * @param frame parent frame of this dialog
	 * @return status, which is either <code>ABORTED</code> or <code>DATA_READ</code>
	 * @throws Exception
	 */
	public int showCaptureDialog(JFrame frame) throws Exception {
		status = ABORTED;
		initDialog(frame);
		setDialogEnabled(true);
		dialog.show();
		return status;
	}

	/**
	 * Displays the device controller dialog with disabled configuration, starting capture immediately.
	 * 
	 * @param frame parent frame of this dialog
	 * @return status, which is either <code>ABORTED</code> or <code>DATA_READ</code>
	 * @throws Exception
	 */
	public int showCaptureProgress(JFrame frame) throws Exception {
		status = ABORTED;
		initDialog(frame);
		startCapture();
		dialog.show();
		return status;
	}

	private Thread worker;
	private Timer timer;
	
	private JComboBox portSelect;
	private JComboBox speedSelect;
	private JComboBox sizeSelect;
	private JComboBox ratioSelect;
	private JCheckBox filterEnable;
	private JCheckBox triggerEnable;
	private JCheckBox[] triggerMask;
	private JCheckBox[] triggerValue;
	private JCheckBox[] channelGroup;
	private JProgressBar progress;
	private JButton captureButton;
	
	private JDialog dialog;
	private Device device;
	private CapturedData capturedData;
	
	private int status;
	
	private static final long serialVersionUID = 1L;
}

==> la-src-0.6.tar.bz2: LogicAnalyzer/client/org/sump/analyzer/DeviceController.java <==
/*
 *  Copyright (C) 2006 Michael Poppitz
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 *
 */
package org.sump.analyzer;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.Timer;

// TODO: when the dialog is closed using the window decoration's close function, close() is not called

/**
 * GUI Component that allows the user to control the device and start captures.
 * <p>
 * Its modelled after JFileChooser and should allow for non-dialog implementations
 * making it somewhat reusable.
 * 
 * @version 0.6
 * @author Michael "Mr. Sump" Poppitz
 *
 */
public class DeviceController extends JComponent implements ActionListener, Runnable {
	/** dialog showing and waiting for user action */
	public final static int IDLE = 0;
	/** capture currently running */
	public final static int RUNNING = 1;
	/** capture / dialog aborted by user */
	public final static int ABORTED = 2;
	/** capture finished */
	public final static int DONE = 3;
	
	/**
	 * Creates an array of check boxes, adds it to the device controller and returns it.
	 * @param label label to use on device controller component
	 * @return array of created check boxes
	 */
	private JCheckBox[] createChannelList(JPanel pane, String label) {
		JCheckBox[] boxes = new JCheckBox[32];

		for (int row = 0; row < 4; row++) {
			if (row == 0)
				pane.add(new JLabel(label));
			else
				pane.add(new JLabel());
	
			Container container = new Container();
			container.setLayout(new GridLayout(1, 8));
	
			for (int col = 0; col < 8; col++) {
				JCheckBox box = new JCheckBox();
				box.setEnabled(false);
				container.add(box);
				boxes[row * 8 + col] = box;
			}
	
			pane.add(container);
		}
		return (boxes);
	}
	
	private static GridBagConstraints createConstraints(int x, int y, int w, int h, double wx, double wy) {
	 	GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
	        gbc.insets = new Insets(2, 2, 2, 2);
	 	gbc.gridx = x; gbc.gridy = y;
		gbc.gridwidth = w; gbc.gridheight = h;
	        gbc.weightx = wx; gbc.weighty = wy;
	        return (gbc);
	}
	
	/**
	 * Constructs device controller component.
	 *
	 */
	public DeviceController() {
		super();
		setLayout(new GridBagLayout());
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		device = new Device();

		JPanel connectionPane = new JPanel();
		connectionPane.setLayout(new GridLayout(2, 2, 5, 5));
		connectionPane.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder("Connection Settings"),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)
		));		String[] ports = Device.getPorts();
		portSelect = new JComboBox(ports);
		connectionPane.add(new JLabel("Analyzer Port:"));
		connectionPane.add(portSelect);

		String[] portRates = {
			"115200bps (LL)",
			"57600bps (LH)",
			"38400bps (HL)",
			"19200bps (HH)"
		};
		portRateSelect = new JComboBox(portRates);
		connectionPane.add(new JLabel("Port Speed (SW1,SW0):"));
		connectionPane.add(portRateSelect);

		add(connectionPane, createConstraints(0, 0, 2, 1, 1.0, 0.5));
		
		JPanel settingsPane = new JPanel();
		settingsPane.setLayout(new GridLayout(15, 2, 5, 5));
		settingsPane.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder("Analyzer Settings"),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)
		));
		
		String[] sources = {
			"Internal",
			"External / Rising",
			"External / Falling"
		};
		sourceSelect = new JComboBox(sources);
		sourceSelect.addActionListener(this);
		settingsPane.add(new JLabel("Samplíng Clock:"));
		settingsPane.add(sourceSelect);
		
		String[] speeds = {
			"200MHz",
			"100MHz", "50MHz", "20MHz", "10MHz", "5MHz", "2MHz", "1MHz",
			"500kHz", "200kHz", "100kHz", "50kHz", "20kHz", "10kHz",
			"1kHz", "500Hz", "200Hz", "100Hz", "50Hz", "20Hz", "10Hz"
		};
		speedSelect = new JComboBox(speeds);
		speedSelect.setSelectedIndex(1);
		speedSelect.addActionListener(this);
		settingsPane.add(new JLabel("Sampling Rate:"));
		settingsPane.add(speedSelect);

		Container groups = new Container();
		groups.setLayout(new GridLayout(1, 4));
		channelGroup = new JCheckBox[4];
		for (int i = 0; i < channelGroup.length; i++) {
			channelGroup[i] = new JCheckBox(Integer.toString(i));
			channelGroup[i].setSelected(true);
			groups.add(channelGroup[i]);
		}
		settingsPane.add(new JLabel("Channel Groups:"));
		settingsPane.add(groups);
		
		String[] sizes = {
			"256K", "128K", "64K", "32K", "16K", "8K", "4K",
			"2K", "1K", "512", "256", "128", "64"
		};
		sizeSelect = new JComboBox(sizes);
		sizeSelect.setSelectedIndex(7);
		settingsPane.add(new JLabel("Recording Size:"));
		settingsPane.add(sizeSelect);

		filterEnable = new JCheckBox("Enable");
		filterEnable.setSelected(true);
		filterEnable.setEnabled(false);
		settingsPane.add(new JLabel("Noise Filter: "));
		settingsPane.add(filterEnable);

		triggerEnable = new JCheckBox("Enable");
		triggerEnable.addActionListener(this);
		settingsPane.add(new JLabel("Trigger: "));
		settingsPane.add(triggerEnable);
		
		String[] ratios = {"0/100", "25/75", "50/50", "75/25", "100/0"};
		ratioSelect = new JComboBox(ratios);
		ratioSelect.setSelectedIndex(2);
		settingsPane.add(new JLabel("Before/After Ratio: "));
		settingsPane.add(ratioSelect);

		triggerMask = createChannelList(settingsPane, "Trigger Channels (0..31):");
		triggerValue = createChannelList(settingsPane, "Trigger Values (0..31):");

		add(settingsPane, createConstraints(0, 1, 2, 1, 1.0, 0.5));

		JPanel progressPane = new JPanel();
		progressPane.setLayout(new BorderLayout());
		progressPane.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder("Progress"),
				BorderFactory.createEmptyBorder(5, 5, 5, 5)
		));
		progress = new JProgressBar(0, 100);
		progressPane.add(progress, BorderLayout.CENTER);
		add(progressPane, createConstraints(0, 2, 2, 1, 1.0, 0));

		captureButton = new JButton("Capture");
		captureButton.addActionListener(this);
		add(captureButton, createConstraints(0, 3, 1, 1, 0.5, 0));

		JButton cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		add(cancel, createConstraints(1, 3, 1, 1, 0.5, 0));

		capturedData = null;
		timer = null;
		worker = null;
		status = IDLE;
	}

	/**
	 * Internal method that initializes a dialog and add this component to it.
	 * @param frame owner of the dialog
	 */
	private void initDialog(JFrame frame) {
		// check if dialog exists with different owner and dispose if so
		if (dialog != null && dialog.getOwner() != frame) {
			dialog.dispose();
			dialog = null;
		}
		// if no valid dialog exists, create one
		if (dialog == null) {
			dialog = new JDialog(frame, "Capture", true);
			dialog.getContentPane().add(this);
			dialog.pack();
			dialog.setResizable(false);
		}
		// reset progress bar
		progress.setValue(0);

		// sync dialog status with device
		updateFields();
	}

	/**
	 * Return the device data of the last successful run.
	 * 
	 * @return device data
	 */
	public CapturedData getDeviceData() {
		return (capturedData);
	}

	/**
	 * Extracts integers from strings regardless of trailing trash.
	 * 
	 * @param s string to be parsed
	 * @return integer value, 0 if parsing fails
	 */
	private int smartParseInt(String s) {
		int val = 0;
	
		try {
			for (int i = 1; i <= s.length(); i++)
				val = Integer.parseInt(s.substring(0, i));
		} catch (NumberFormatException E) {}
		
		return (val);
	}

	/**
	 * Starts capturing from device. Should not be called externally.
	 */
	public void run()  {
		String value = (String)portSelect.getSelectedItem();
		// TODO: need to check if attach was successful
		device.attach(value, smartParseInt((String)portRateSelect.getSelectedItem()));
		
		value = (String)sizeSelect.getSelectedItem();
		int s = smartParseInt(value);
		if (value.indexOf("K") > 0)
			s *= 1024;
		System.out.println("Size: " + s);
		device.setSize(s);
		
		value = (String)ratioSelect.getSelectedItem();
		double r = 0.5;
		if (value.equals("100/0")) r = 0;
		else if (value.equals("25/75")) r = 0.75;
		else if (value.equals("50/50")) r = 0.5;
		else if (value.equals("75/25")) r = 0.25;
		else if (value.equals("0/100")) r = 1;
		System.out.println("Ratio: " + r);
		device.setRatio(r);
			
		device.setFilterEnabled(filterEnable.isSelected());
		int m = 0;
		int v = 0;
		for (int i = 0; i < 32; i++) {
			if (triggerMask[i].isSelected())
				m |= 1 << i;
			if (triggerValue[i].isSelected())
				v |= 1 << i;				
		}
		device.setTrigger(m, v);

		int enabledChannels = 0;
		for (int i = 0; i < channelGroup.length; i++)
			if (channelGroup[i].isSelected())
				enabledChannels |= 0xff << (8 * i);
		device.setEnabledChannels(enabledChannels);

		status = RUNNING;
		
		try {
			System.out.println("Run started");
			capturedData = device.run();
			System.out.println("Run completed");
			status = DONE;
		} catch (Exception ex) {
			// TODO: could make sense to also return half read captures if array length is corrected
			capturedData = null;
			status = ABORTED;
			System.out.println("Run aborted");
			if (!(ex instanceof InterruptedException))
				ex.printStackTrace(System.out);
		}
		device.detach();
	}
	
	/**
	 * Sets the enabled state of all available trigger check boxes and the ratio select.
	 * @param enable <code>true</code> to enable trigger configuration fields, <code>false</code> to disable them
	 */
	private void setTriggerEnabled(boolean enable) {
		int channels = device.getAvailableChannelCount();
		
		for (int i = 0; i < channels; i++) {
			triggerMask[i].setEnabled(enable);
			triggerValue[i].setEnabled(enable);
		}
		for (int i = channels; i < 32; i++) {
			triggerMask[i].setEnabled(false);
			triggerValue[i].setEnabled(false);
		}
		ratioSelect.setEnabled(enable);
	}
	
	/**
	 * Sets the enabled state of all configuration components of the dialog.
	 * @param enable <code>true</code> to enable components, <code>false</code> to disable them
	 */
	private void setDialogEnabled(boolean enable) {
		triggerEnable.setEnabled(enable);
		captureButton.setEnabled(enable);
		portSelect.setEnabled(enable);
		portRateSelect.setEnabled(enable);
		speedSelect.setEnabled(enable);
		sizeSelect.setEnabled(enable);
		updateFields(enable);
	}

	/** activates / deactivates dialog options according to device status */
	public void updateFields() {
		updateFields(true);
	}

	/** activates / deactivates dialog options according to device status */
	private void updateFields(boolean enable) {
		triggerEnable.setSelected(device.isTriggerEnabled());
		if (triggerEnable.isSelected() || !enable)
			setTriggerEnabled(enable);
		setTriggerEnabled(device.isTriggerEnabled());
		filterEnable.setEnabled(device.isFilterAvailable() && enable);
		for (int i = 0; i < channelGroup.length; i++)
			channelGroup[i].setEnabled(enable && (i < device.getAvailableChannelCount() / 8));
		speedSelect.setEnabled(device.getClockSource() == Device.CLOCK_INTERNAL);
	}
	
	/**
	 * Properly closes the dialog.
	 * This method makes sure timer and worker thread are stopped before the dialog is closed.
	 *
	 */
	private void close() {
		if (timer != null) {
			timer.stop();
			timer = null;
		}
		if (worker != null) {
			device.stop(); // lets hope no one gets here before device.run() is called
			worker.interrupt();
			worker = null;
		}
		dialog.hide();
	}
	
	/**
	 * Starts the capture thread.
	 */
	private void startCapture() {
		try {
			setDialogEnabled(false);
			timer = new Timer(100, this);
			worker = new Thread(this);
			timer.start();
			worker.start();
		} catch(Exception E) {
			E.printStackTrace(System.out);
		}
	}
	
	/**
	 * Handles all action events for this component.
	 */ 
	public void actionPerformed(ActionEvent event) {
		Object o = event.getSource();
		String l = event.getActionCommand();
		
		if (o == timer) {
			if (status == DONE) {
				close();
			} else if (status == ABORTED) {
				timer.stop();
				JOptionPane.showMessageDialog(this,
					"Error while trying to communicate with device.\n\n"
					+ "Make sure the device is:\n"
					+ " - connected to the specified port\n"
					+ " - turned on and properly programmed\n"
					+ " - set to the selected transfer rate\n",
					"Communication Error",
					JOptionPane.ERROR_MESSAGE
				);
				setDialogEnabled(true);
			} else {
				if(device.isRunning())
					progress.setValue(device.getPercentage());
			}
		} else {
		
			if (o == triggerEnable) {
				device.setTriggerEnabled(triggerEnable.isSelected());
				updateFields();

			} else if (o == sourceSelect) {
				String value = (String)sourceSelect.getSelectedItem();
				if (value.equals("Internal")) {
					device.setClockSource(Device.CLOCK_INTERNAL);
				} else {
					if (value.equals("External / Rising"))
						device.setClockSource(Device.CLOCK_EXTERNAL_RISING);
					else 
						device.setClockSource(Device.CLOCK_EXTERNAL_FALLING);
				}
				System.out.println("Clock Source: " + device.getClockSource());
				updateFields();

			} else if (o == speedSelect) {
				String value = (String)speedSelect.getSelectedItem();
				int f = smartParseInt(value);
				if (value.indexOf("M") > 0)
					f *= 1000000;
				else if (value.indexOf("k") > 0)
					f *= 1000;
				System.out.println("Rate: " + f);
				device.setRate(f);
				updateFields();

			} else if (l.equals("Capture")) {
				startCapture();

			} else {
				close();
			}
		}
	}
	
	/**
	 * Displays the device controller dialog with enabled configuration portion and waits for user input.
	 * 
	 * @param frame parent frame of this dialog
	 * @return status, which is either <code>ABORTED</code> or <code>DONE</code>
	 * @throws Exception
	 */
	public int showCaptureDialog(JFrame frame) throws Exception {
		status = IDLE;
		initDialog(frame);
		setDialogEnabled(true);
		dialog.show();
		return status;
	}

	/**
	 * Displays the device controller dialog with disabled configuration, starting capture immediately.
	 * 
	 * @param frame parent frame of this dialog
	 * @return status, which is either <code>ABORTED</code> or <code>DONE</code>
	 * @throws Exception
	 */
	public int showCaptureProgress(JFrame frame) throws Exception {
		status = IDLE;
		initDialog(frame);
		startCapture();
		dialog.show();
		return status;
	}

	private Thread worker;
	private Timer timer;
	
	private JComboBox portSelect;
	private JComboBox portRateSelect;
	private JComboBox sourceSelect;
	private JComboBox speedSelect;
	private JComboBox sizeSelect;
	private JComboBox ratioSelect;
	private JCheckBox filterEnable;
	private JCheckBox triggerEnable;
	private JCheckBox[] triggerMask;
	private JCheckBox[] triggerValue;
	private JCheckBox[] channelGroup;
	private JProgressBar progress;
	private JButton captureButton;
	
	private JDialog dialog;
	private Device device;
	private CapturedData capturedData;
	
	private int status;
	
	private static final long serialVersionUID = 1L;
}
